#!/usr/bin/perl
#
#	Copyright 2002 Matthew Colyer
#	GPL V2.0 see COPYING
#
#	Converts Calendar data from Evolution to the Zaurus's format.

use strict;
use Date::Parse;
use Net::ICal;
use XML::Writer;
use IO;

sub e2z_cal {
	my $evo_file = shift;
	my $z_file = shift;
	
	#Open the Evolution calendar file
	open CALFILE, "<$evo_file" or 
			die "Unable to open calendar file\n";
	undef $/;
	my $cal = Net::ICal::Component->new_from_ical (<CALFILE>) ;
	close CALFILE;
	
	#Open the Zaurus todo file
	my $output = new IO::File(">$z_file");
	my $writer = new XML::Writer(OUTPUT => $output, DATA_MODE => 1);
	
	my $event_list = $cal->events;
	my ($location, $uid, $alarm, $alarm_type, $note);
	
	#Start the xml file
	$writer->xmlDecl("UTF-8");
	$writer->doctype("DATEBOOK");
	$writer->startTag("DATEBOOK");
	$writer->startTag("RIDMax");
	$writer->characters(scalar(@$event_list));
	$writer->endTag("RIDMax");
	$writer->startTag("events");
	
	#Okay here is the loop which does all the dirty work
	foreach my $event (@$event_list) {
		#The first part of the loop processes the data
		# so that we are able to add it all in one function
		# call to the xml writer.
		
		#This little part is to undo the changes we made
		# to the unique ids so the Evolution calendar didn't
		# choke.
		$event->uid =~ /Z(.*)@/;
		$uid = $1;
		
		#Perl won't let us reference and undefined variable 
		# (imagine that) so we have to check to make sure they
		# exist first.
		if (defined $event->location){
			$location = scalar $event->location->{content};
		}else{
			$location = "";
		}
		if (defined $event->description){
			$note = scalar $event->description->{content};
		}else{
			$note = "";
		}
		
		#Okay there are two main types of entries for the zaurus
		# ones that have an alarm and ones that do not.
		# There really is a third main difference, a repetative
		# event but I don't feel like writing the code for it.
		
		if (defined $event->alarms){
			
			#get the related alarm event
			$alarm = $event->alarms;
			$alarm = @$alarm[0];
			
			#This is broken because NET::ICal does not correct
			#deduce what the action is from an outlook vcard file
			#It defaults to LOUD.
			if ($alarm->action eq "DISPLAY"){
				$alarm_type = "silent";
			}else{
				$alarm_type = "loud";			
			}
			
			$writer->emptyTag("event", 
			            "description" => $event->summary,
			            "location" => $location,
			            "categories" => "",
			            "uid" => $uid,
			            "rid" => "",
			            "rinfo" => "",
			            "alarm" => $alarm->trigger->content->nsecs/60,
			            "sound" => $alarm_type,
			            "start" => str2time(scalar $event->dtstart->as_gmtime),
			            "end" => str2time(scalar $event->dtend->as_gmtime),
						"note" => $note,
			            );
			
		}else{
			$writer->emptyTag("event", 
			            "description" => $event->summary,
			            "location" => $location,
			            "categories" => "",
			            "uid" => $uid,
			            "rid" => "",
			            "rinfo" => "",
			            "start" => str2time(scalar $event->dtstart->as_gmtime),
			            "end" => str2time(scalar $event->dtend->as_gmtime),
			            "note" => $note,
			            );
		}
	}
	
	#Okay were all done adding elements.
	# All that is left is to close up the document.
	
	#BTW wouldn't it be nice if trolltech actually had
	# some kind of standard between its todo list, calendar
	# and contact xml? They all have varying numbers of root
	# tags and as well as different methods of line returns and
	# capitalization for attributes.
	#
	#	HINT, HINT, maybe if they open sourced qtopiadesktop
	#	people might fix this uglyness.
		
	$writer->endTag("events");
	$writer->endTag("DATEBOOK");
	$writer->end();
	$output->close();
}

#Leave this in here
1;