#!/usr/bin/perl

use strict ;
use BerkeleyDB ;
use vars qw( %contact_db) ;
use IO;
use XML::Writer;

sub e2z_contacts{
	my $evo_file = shift;
	my $z_file = shift;

	tie %contact_db, "BerkeleyDB::Hash",
	    -Filename => $evo_file
	    or die "Cannot open file $evo_file: $! $BerkeleyDB::Error\n" ;
	
	#Open the Zaurus contact file
	my $output = new IO::File(">$z_file");
	my $writer = new XML::Writer(OUTPUT => $output, DATA_MODE => 1);
	
	#Start the xml file
	$writer->xmlDecl("UTF-8");
	$writer->doctype("AddressBook");
	$writer->startTag("AddressBook");
	$writer->startTag("RIDMax");
	$writer->characters(scalar(%contact_db)-1);
	$writer->endTag("RIDMax");
	$writer->startTag("Groups");
	$writer->endTag("Groups");
	$writer->startTag("Contacts");
	
	foreach my $contact ( keys %contact_db ){
		$_ = $contact_db{$contact};
		s/\r\n/\n/g;
		s/\0//g;
		if ($_ =~ /UID/m){
			parse_card($_, $writer);
		}
	}
	$writer->endTag("Contacts");
	$writer->endTag("AddressBook");
	$writer->end();
	$output->close();
	
	untie %contact_db ;
}

sub parse_card{
	my $string = shift;
	my $writer = shift;
	my $vcard;

	foreach my $ln ( split(/\n/,$string )){
		  # FN
		  if ($ln =~ /^F/) {
			$ln =~ /^FN:(.*)$/;
			$vcard->{fn} = $1;
		  }
		
		  # N
		  elsif ($ln =~ /^N:/i) {
			# Family Name, Given Name, Additional Names, 
			# Honorific Prefixes, and Honorific Suffixes.
			$ln =~ /^N:([^;]+)?;([^;]+)?;([^;]+)?;([^;]+)?;([^;]+)?$/;
			$vcard->{n} = {family=>$1,given=>$2,other=>$3,prefixes=>$4,suffixes=>$5};
		  }
		
		  # NICKNAME
		  elsif ($ln =~ /^NI/) {
			$ln =~ /^NICKNAME:(.*)$/;
			$vcard->{nickname} = $1;
		  }
		
		  # BDAY
		  elsif ($ln =~ /^BD/) {
			$ln =~ /^BDAY:(.*)$/;
			$vcard->{bday} = $1;
		  }
		
		  # ADR
		  if ($ln =~ /^AD/) {
			$ln =~ /^ADR;([^:]+)?:([^;]+)?;([^;]+)?;([^;]+)?;([^;]+)?;([^;]+)?;([^;]+)?;([^;]+)?$/i;
			push @{$vcard->{adr}} , {"type"=>$1,pobox=>$2,extadr=>$3,street=>$4,city=>$5,zip=>$6,state=>$7,country=>$8};
		  }
		
		  # TEL
		  elsif ($ln =~ /^TE/) {
			$ln =~ /^TEL;([^:]+)?:(.*)$/;
			push @{$vcard->{tel}},{"type"=>$1,number=>$2};
		  }
		
		  # EMAIL
		  elsif ($ln =~ /^EM/) {
			$ln =~ /^EMAIL;([^:]+)?:(.*)$/;
			push @{$vcard->{email}},{"type"=>$1,address=>$2};
		  }
		
		  # TITLE
		  elsif ($ln =~ /^TI/) {
			$ln =~ /^TITLE:(.*)$/;
			$vcard->{title} = $1;
		  }
		
		  # ORG
		  elsif ($ln =~ /^O/) {
			$ln =~ /^ORG:([^;]+);([^;]+);(.*)$/;
			$vcard->{org} = {name=>$1,unit=>$2};
		  }
		
		  # CATEGORIES
		  elsif ($ln =~ /^CA/) {
			$ln =~ /^CATEGORIES:(.*)$/;
			$vcard->{categories} = [split(",",$1)];
		  }
		
		  # NOTE
		  elsif ($ln =~ /^NO/) {
			$ln =~ /^NOTE:(.*)$/;
			$vcard->{note} = $1;
		  }
		
		  # UID
		  elsif ($ln =~ /^UI/) {
			$ln =~ /^UID:(.*)$/;
			$vcard->{uid} = $1;
		  }
		
		  # URL
		  elsif ($ln =~ /^UR/) {
			$ln =~ /^URL:(.*)$/;
			push @{$vcard->{url}},$1;
		  }
		
		  # X-CUSTOM
		  elsif ($ln =~ /^X/) {
			$ln =~ /^X-EVOLUTION-([^:]+):(.*)$/;
			push @{$vcard->{'x-custom'}}, {$1=>$2};
		  }
	}
	my ($emails, $file_as, $h_phone, $h_cell, $h_fax) = "";
	my ($w_fax, $w_phone, $w_cell, $w_pager) = "";

	foreach (@{$vcard->{'x-custom'}}) {
		if (defined $_->{'FILE-AS'}){
			$file_as = $_->{'FILE-AS'};
		}
	}
	foreach (@{$vcard->{'email'}}) {
		$emails .= $_->{address}." ";
	}
	foreach (@{$vcard->{'tel'}}) {
		if ($_->{"type"} eq "HOME"){
			$h_phone = $_->{number};
		}
		if ($_->{"type"} eq "CELL"){
			$h_cell = $_->{number};
		}
		if ($_->{"type"} eq "HOME;FAX"){
			$h_fax = $_->{number};
		}
		if ($_->{"type"} eq "WORK;VOICE:"){
			$w_phone = $_->{number};
		}
		if ($_->{"type"} eq "WORK;FAX"){
			$w_fax = $_->{number};
		}
		if ($_->{"type"} eq "WORK;CELL"){
			$w_cell = $_->{number};
		}
		if ($_->{"type"} eq "WORK;PAGER"){
			$w_pager = $_->{number};
		}
	}
	my ($h_str, $h_city, $h_zip, $h_st, $h_co) =" ";
	my ($w_str, $w_city, $w_zip, $w_st, $w_co) = "";
	foreach (@{$vcard->{'adr'}}) {
		if ($_->{"type"} eq "HOME"){
			$h_str = $_->{street};
			$h_city = $_->{city};
			$h_zip = $_->{zip};
			$h_st = $_->{state};
			$h_co = $_->{country};
		}
		if ($_->{"type"} eq "WORK"){
			$w_str = $_->{street};
			$w_city = $_->{city};
			$w_zip = $_->{zip};
			$w_st = $_->{state};
			$w_co = $_->{country};
		}
	}
	my ($spouse, $anniversary) = "";
	foreach (@{$vcard->{'x-custom'}}) {
		if (defined $_->{'SPOUSE'}){
			$spouse= $_->{'SPOUSE'};
		}
		if (defined $_->{'ANNIVERSARY'}){
			$anniversary = $_->{'ANNIVERSARY'};
		}
	}
	$vcard->{uid} =~ /.*-id(-.*)/;
	$writer->emptyTag("Contact", 
	        "FirstName" => $vcard->{n}->{given},
	        "MiddleName" => $vcard->{n}->{family},
	        "LastName" => $vcard->{n}->{other},
	        "FileAs" => $file_as,
	        "JobTitle" => $vcard->{title},
	        "Department" => $vcard->{dept}->{name},
	        "Company" => $vcard->{org}->{name},
	        "BusinessPhone" => $w_phone,
	        "BusinessFax" => $w_fax,
	        "BusinessMobile" => $w_cell,
	        "Emails" => $emails,
	        "HomePhone" => $h_phone,
	        "HomeFax" => $h_fax,
	        "HomeMobile" => $h_cell,
	        "BusinessStreet" => $w_str,
	        "BusinessCity" => $w_city,
	        "BusinessZip" => $w_zip,
	        "BusinessState" => $w_st,
	        "BusinessCountry" => $w_co,
	        "BusinessWebPage" => pop(@{$vcard->{url}}),
	        "HomeStreet" => $h_str,
	        "HomeCity" => $h_city,
	        "HomeZip" => $h_zip,
	        "HomeState" => $h_st,
	        "HomeCountry" => $h_co,
	        "Spouse" => $spouse,
	        "Birthday" => $vcard->{bday},
	        "Anniversary" => $anniversary,
	        "Nickname" => $vcard->{nickname},
	        "Notes" => $vcard->{note},
	        "rid" => "",
	        "rinfo" => "",
	        "Categories" => "",
			"Uid" => $1,
	        );
}

#Leave this in here
1;