#!/usr/bin/perl
#
#	Copyright 2002 Matthew Colyer
#	GPL V2.0 see COPYING
#
#	Converts Task data from Evolution to the Zaurus's format.

use strict;
use Net::ICal;
use XML::Writer;
use IO;

sub e2z_tasks(){
	my $evo_file = shift;
	my $z_file = shift;
	
	#Open the Evolution todo file
	open TASKFILE, "<$evo_file" or 
			die "Unable to open task file\n";
	undef $/;
	my $cal = Net::ICal::Component->new_from_ical (<TASKFILE>) ;
	close TASKFILE;
	
	#Open the Zaurus todo file
	my $output = new IO::File(">$z_file");
	my $writer = new XML::Writer(OUTPUT => $output, DATA_MODE => 1);
	
	my $todo_list = $cal->todos;
	my ($status, $priority);
	
	#Start the xml file
	$writer->doctype("Tasks");
	$writer->startTag("Tasks");
	$writer->startTag("RIDMax");
	$writer->characters(scalar(@$todo_list));
	$writer->endTag("RIDMax");
	
	#Okay here is the loop which does all the dirty work
	foreach my $item (@$todo_list) {
		#The first part of the loop processes the data
		# so that we are able to add it all in one function
		# call to the xml writer.
		
		if ($item->status eq "COMPLETED"){
			$status = "1";
		}else{
			$status = "0";
		}
		
		#Evolution's Prioritys include 3,5,7
		#Zaurus's range from 0-5
		#You do the math....
		#Its a bit of a hack since you don't get as fine grain control
		if ($item->priority eq "3"){
			$priority = 2;
		}
		if ($item->priority eq "5"){
			$priority = 3;
		}
		if ($item->priority eq "7"){
			$priority = 4;
		}		
		
		#Okay there are two main types of entries for the Zaurus
		# ones that have a due date and ones that do not.
		
		if (defined $item->due) {
			$writer->emptyTag("Task", 
		                "Completed" => $status,
		                "HasDate" => "1",
		                "Priority" => $priority,
		                "Categories" => "",
		                "Description" => $item->summary,
		                "DateYear" => substr($item->due->as_ical_value,0,4),
		                "DateMonth" => substr($item->due->as_ical_value,4,2),
		                "DateDay" => substr($item->due->as_ical_value,6,2),
		                "Uid" => $item->uid,
		                "Rid" => "",
		                "Rinfo" => $item->sequence
		                );
		}else{
			$writer->emptyTag("Task", 
		                "Completed" => $status,
		                "HasDate" => "0",
		                "Priority" => $priority,
		                "Categories" => "",
		                "Description" => $item->summary,
		                "Uid" => $item->uid,
		                "Rid" => "",
		                "Rinfo" => $item->sequence
		                );
	    }
	}
	
	#Okay were all done adding elements.
	# All that is left is to close up the document.
	$writer->endTag();
	$writer->end();
	$output->close();
}
#Leave this in here
1;