#!/usr/bin/perl
#
#	Copyright 2002 Matthew Colyer
#	GPL V2.0 see COPYING
#
#	Converts contact data from Zaurus to Evolution's format.

use strict ;
use vars qw( ) ;
use XML::Parser;

sub z2e_contacts{
	my $evo_file = shift;
	my $z_file = shift;
	
	unlink $evo_file;
	tie %contact_db, "BerkeleyDB::Hash",
	    -Filename => $evo_file,
		-Flags    => DB_CREATE
	    or die "Cannot open file $evo_file: $! $BerkeleyDB::Error\n" ;
	$contact_db{"PAS-DB-VERSION\0"} = "0.2\0";
	my $parser = new XML::Parser(ErrorContext => 2);
	$parser->setHandlers(Start => \&contact_element_handler);
	$parser->parsefile($z_file);
	
	untie %contact_db ;
}

sub contact_element_handler
{
	my $expat = shift; my $element = shift;
	my %vcard;
	my $email;
	my $file;
	my $vcard_text;
	
	# Gather all the attributes of the element
    while (@_) {
    	my $attr = shift;
    	my $val = shift;
    	
    	$vcard{$attr} = $val;		
    }
    
    # if it is a contact element then make a vcard entry and add it to the database
    if (!($vcard{'Uid'} eq "")){
    	
	    $vcard_text = "BEGIN:VCARD\r\n";
	    $vcard_text .= "X-EVOLUTION-FILE-AS:".$vcard{'LastName'}.", ".$vcard{'FirstName'}."\r\n";
	    $vcard_text .= "FN:".$vcard{'FileAs'}."\r\n";
	    $vcard_text .= "N:".$vcard{'LastName'}.";".$vcard{'FirstName'}.";".$vcard{'MiddleName'}.";;\r\n";
	    if (!($vcard{'Birthday'} eq "")){
	   	$vcard_text .= "BDAY:".$vcard{'Birthday'}."\r\n";
	    }
	    foreach $email (split(/ /,$vcard{'Emails'})){
	    	$vcard_text .= "EMAIL;INTERNET:".$email."\r\n";
	    }
	    if (!($vcard{'HomePhone'} eq "")){
	    	$vcard_text .= "TEL;HOME:".$vcard{'HomePhone'}."\r\n";
	    }
	    if (!($vcard{'HomeMobile'} eq "")){
	    	$vcard_text .= "TEL;CELL:".$vcard{'HomeMobile'}."\r\n";
	    }
	    if (!($vcard{'HomeFax'} eq "")){
	    	$vcard_text .= "TEL;HOME;FAX:".$vcard{'HomeFax'}."\r\n";
	    }
	    if (!($vcard{'Company'} eq "" or $vcard{'Department'} eq "")){
	    	$vcard_text .= "ORG:".$vcard{'Company'}.";".$vcard{'Department'}.";\r\n";
	    }
	    if (!($vcard{'BusinessPhone'} eq "")){
		    $vcard_text .= "TEL;WORK;VOICE:".$vcard{'BusinessPhone'}."\r\n";
		}
		if (!($vcard{'BusinessFax'} eq "")){
		    $vcard_text .= "TEL;WORK;FAX:".$vcard{'BusinessFax'}."\r\n";
		}
		if (!($vcard{'BusinessMobile'} eq "")){
		    $vcard_text .= "TEL;WORK;CELL:".$vcard{'BusinessMobile'}."\r\n";
		}
		if (!($vcard{'BusinessPager'} eq "")){
		    $vcard_text .= "TEL;WORK;PAGER:".$vcard{'BusinessPager'}."\r\n";
		}
		if (!($vcard{'Nickname'} eq "")){
		    $vcard_text .= "NICKNAME:".$vcard{'Nickname'}."\r\n";
		}
		if (!($vcard{'Spouse'} eq "")){
		    $vcard_text .= "X-EVOLUTION-SPOUSE:".$vcard{'Spouse'}."\r\n";
		}
		if (!($vcard{'Anniversary'} eq "")){
		    $vcard_text .= "X-EVOLUTION-ANNIVERSARY:".$vcard{'Anniversary'}."\r\n";
		}
		if (!($vcard{'Notes'} eq "")){
		    $vcard_text .= "Note:".$vcard{'Notes'}."\r\n";
		}
		if (!($vcard{'JobTitle'} eq "")){
		    $vcard_text .= "TITLE:".$vcard{'JobTitle'}."\r\n";
		}
		if (!($vcard{'HomeWebPage'} eq "")){
		    $vcard_text .= "URL:".$vcard{'HomeWebPage'}."\r\n";
		}
		if (!($vcard{'HomeStreet'} eq "" 
				or $vcard{'HomeCity'} eq ""
				or $vcard{'HomeZip'} eq ""
				or $vcard{'HomeState'} eq ""
				or $vcard{'HomeCountry'} eq "")){
			$vcard_text .= "ADR;HOME:;;".$vcard{'HomeStreet'}.";".$vcard{'HomeCity'}.
							";".$vcard{'HomeZip'}.";".$vcard{'HomeState'}.";".$vcard{'HomeCountry'}."\r\n";
			$vcard_text .= "LABEL;QUOTED-PRINTABLE;HOME;PREF:".$vcard{'HomeStreet'}."=0A".$vcard{'HomeCity'}.
							", ".$vcard{'HomeZip'}." ".$vcard{'HomeState'}."=0A".$vcard{'HomeCountry'}."\r\n";
		}
		if (!($vcard{'BusinessStreet'} eq "" 
				or $vcard{'BusinessCity'} eq ""
				or $vcard{'BusinessZip'} eq ""
				or $vcard{'BusinessState'}eq ""
				or $vcard{'BusinessCountry'} eq "") ){

			$vcard_text .= "ADR;WORK:;;".$vcard{'BusinessStreet'}.";".$vcard{'BusinessCity'}.";".
							$vcard{'BusinessZip'}.";".$vcard{'BusinessState'}.";".$vcard{'BusinessCountry'}."\r\n";
			$vcard_text .= "LABEL;QUOTED-PRINTABLE;WORK;PREF:".$vcard{'BusinessState'}."=0A".$vcard{'BusinessCity'}.
							", ".$vcard{'BusinessZip'}." ".$vcard{'BusinessState'}."=0A".$vcard{'BusinessCountry'}."\r\n";
	    }
	    $vcard_text .= "UID:zaurus-id".$vcard{'Uid'}."\r\n";
	    $vcard_text .= "END:VCARD\r\n\r\n\0";
		$contact_db{"zaurus-id".$vcard{'Uid'}."\0"} = $vcard_text;
	}
}

#Leave this in here
1;